import json
import os


def load_fixture(filename):
    """Load a fixture."""
    with open(
        os.path.join(os.path.dirname(__file__), "fixtures", filename), encoding="utf-8"
    ) as fp:
        data = json.load(fp)
    return data



from __future__ import annotations

import base64
import time
from typing import TYPE_CHECKING, Optional

import requests
from Crypto.Cipher import AES
from Crypto.Hash import CMAC

from .const import APIGW_URL, IOT_EP, OFFICIALAPI_URL, AuthType
from .helper import CHSesame2MechStatus
from .history import CHSesame2History

if TYPE_CHECKING:
    from .lock import CHSesame2, CHSesame2CMD


class SesameCloud:
    def __init__(self, device: CHSesame2) -> None:
        """Constructs and sends a Request to the cloud.

        Args:
            device (CHSesame2): The device for which you want to query.
        """
        self._device = device

    def requestAPI(
        self, method: str, url: str, json: Optional[dict] = None
    ) -> requests.Response:
        """A Wrapper of `requests.request`.

        Args:
            method (str): HTTP method to use: `GET`, `OPTIONS`, `HEAD`, `POST`, `PUT`, `PATCH`, or `DELETE`.
            url (str): URL to send.
            json (Optional[dict], optional): JSON data for the body to attach to the request. Defaults to `None`.

        Raises:
            RuntimeError: An HTTP error occurred.

        Returns:
            requests.Response: The server's response to an HTTP request.
        """
        try:
            response = requests.request(
                method,
                url,
                json=json,
                auth=self._device.authenticator,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise RuntimeError(e)

        return response

    def getSign(self) -> str:
        """Generates a AES-CMAC tag.

        Returns:
            str: AES-CMAC tag.
        """
        j2 = int(time.time())
        bArr = []
        bArr.append(((j2 >> 8) & 65535) & 0xFF)
        bArr.append(((j2 >> 16) & 65535) & 0xFF)
        bArr.append(((j2 >> 24) & 65535) & 0xFF)
        secret = bytes.fromhex(self._device.getSecretKey())
        cobj = CMAC.new(secret, ciphermod=AES)
        cobj.update(bArr)
        sign = cobj.hexdigest()

        return sign

    def getMechStatus(self) -> CHSesame2MechStatus:
        """Retrives a mechanical status of a device.

        Returns:
            CHSesame2MechStatus: Current mechanical status of the device.
        """
        if self._device.authenticator.login_method == AuthType.WebAPI:
            url = "{}/{}".format(OFFICIALAPI_URL, self._device.getDeviceUUID())
            response = self.requestAPI("GET", url)
            r_json = response.json()
            return CHSesame2MechStatus(dictdata=r_json)
        elif self._device.authenticator.login_method == AuthType.SDK:
            url = "https://{}/things/sesame2/shadow?name={}".format(
                IOT_EP, self._device.getDeviceUUID()
            )

            response = self.requestAPI("GET", url)
            r_json = response.json()
            return CHSesame2MechStatus(rawdata=r_json["state"]["reported"]["mechst"])

    def sendCmd(self, cmd: CHSesame2CMD, history_tag: str = "pysesame3") -> bool:
        """Sends a locking/unlocking command.

        Args:
            cmd (CHSesame2CMD): Lock, Unlock and Toggle.
            history_tag (CHSesame2CMD): The key tag to sent when locking and unlocking.

        Returns:
            bool: `True` if success, `False` if not.
        """
        if self._device.authenticator.login_method == AuthType.WebAPI:
            url = "{}/{}/cmd".format(OFFICIALAPI_URL, self._device.getDeviceUUID())
            sign = self.getSign()
        elif self._device.authenticator.login_method == AuthType.SDK:
            url = "{}/device/v1/iot/sesame2/{}".format(
                APIGW_URL, self._device.getDeviceUUID()
            )
            sign = self.getSign()[0:8]

        payload = {
            "cmd": int(cmd),
            "history": base64.b64encode(history_tag.encode()).decode(),
            "sign": sign,
        }

        try:
            response = self.requestAPI("POST", url, payload)

            return response.ok
        except RuntimeError:
            return False

    def getHistoryEntries(self) -> list[CHSesame2History]:
        """Retrieves the history of all events with a device.

        Returns:
            list[CHSesame2History]: A list of events.
        """
        if self._device.authenticator.login_method == AuthType.WebAPI:
            url = "{}/{}/history?page=0&lg=10".format(
                OFFICIALAPI_URL, self._device.getDeviceUUID()
            )
        elif self._device.authenticator.login_method == AuthType.SDK:
            url = "{}/device/v1/sesame2/{}/history?page=0&lg=10&a={}".format(
                APIGW_URL, self._device.getDeviceUUID(), self.getSign()[0:8]
            )

        ret = []

        response = self.requestAPI("GET", url)
        for entry in response.json():
            ret.append(CHSesame2History(**entry))

        return ret



#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import pytest
from pysesame3.helper import CHProductModel, CHSesame2MechStatus


class TestCHProductModel:
    def test_CHProductModel_raises_exception_on_invalid_model(self):
        with pytest.raises(AttributeError):
            CHProductModel.SS99

    def test_CHProductModel_SS2(self):
        ss2 = CHProductModel.SS2
        assert ss2.deviceModel() == "sesame_2"
        assert ss2.isLocker()
        assert ss2.productType() == 0
        assert ss2.deviceFactory().__name__ == "CHSesame2"

    def test_CHProductModel_WM2(self):
        wm2 = CHProductModel.WM2
        assert wm2.deviceModel() == "wm_2"
        assert not wm2.isLocker()
        assert wm2.productType() == 1
        assert wm2.deviceFactory() is None

    def test_CHProductModel_getByModel_raises_exception_on_invalid_arguments(self):
        with pytest.raises(ValueError):
            CHProductModel.getByModel(123)

    def test_CHProductModel_getByModel_returns_None_for_unknown_model(self):
        assert CHProductModel.getByModel("sesame_99") is None

    def test_CHProductModel_getByModel_returns_SS2(self):
        assert CHProductModel.getByModel("sesame_2") is CHProductModel.SS2

    def test_CHProductModel_getByValue_raises_exception_on_invalid_arguments(self):
        with pytest.raises(ValueError):
            CHProductModel.getByValue("0")

    def test_CHProductModel_getByValue_returns_None_for_unknown_model(self):
        assert CHProductModel.getByValue(999) is None

    def test_CHProductModel_getByValue_returns_SS2(self):
        assert CHProductModel.getByValue(0) is CHProductModel.SS2


class TestCHSesame2MechStatus:
    def test_CHSesame2MechStatus_raises_exception_on_emtry_arguments(self):
        with pytest.raises(ValueError):
            CHSesame2MechStatus()

    def test_CHSesame2MechStatus_raises_exception_on_non_string_argument(self):
        with pytest.raises(TypeError):
            CHSesame2MechStatus(10)

    def test_CHSesame2MechStatus_dictdata_locked(self):
        status = CHSesame2MechStatus(
            dictdata={
                "batteryVoltage": 5.869794721407625,
                "position": 11,
                "CHSesame2Status": "locked",
            }
        )

        assert status.getBatteryPrecentage() == 67
        assert status.getBatteryVoltage() == 5.869794721407625
        assert status.getPosition() == 11
        with pytest.raises(NotImplementedError):
            status.getRetCode()
        with pytest.raises(NotImplementedError):
            status.getTarget()
        assert status.isInLockRange()
        assert not status.isInUnlockRange()
        assert (
            str(status)
            == "CHSesame2MechStatus(Battery=67% (5.87V), isInLockRange=True, isInUnlockRange=False, Position=11)"
        )

    def test_CHSesame2MechStatus_dictdata_unlocked(self):
        status = CHSesame2MechStatus(
            dictdata={
                "batteryVoltage": 5.869794721407625,
                "position": 11,
                "CHSesame2Status": "unlocked",
            }
        )

        assert status.getBatteryPrecentage() == 67
        assert status.getBatteryVoltage() == 5.869794721407625
        assert status.getPosition() == 11
        with pytest.raises(NotImplementedError):
            status.getRetCode()
        with pytest.raises(NotImplementedError):
            status.getTarget()
        assert not status.isInLockRange()
        assert status.isInUnlockRange()
        assert (
            str(status)
            == "CHSesame2MechStatus(Battery=67% (5.87V), isInLockRange=False, isInUnlockRange=True, Position=11)"
        )

    def test_CHSesame2MechStatus_dictdata_various_battery(self):
        status = CHSesame2MechStatus(
            dictdata={
                "batteryVoltage": 0.0,
                "position": 0,
                "CHSesame2Status": "locked",
            }
        )
        assert status.getBatteryPrecentage() == 0
        assert status.getBatteryVoltage() == 0.0

        status2 = CHSesame2MechStatus(
            dictdata={
                "batteryVoltage": 5.5,
                "position": 0,
                "CHSesame2Status": "locked",
            }
        )
        assert status2.getBatteryPrecentage() == 26
        assert status2.getBatteryVoltage() == 5.5

    def test_CHSesame2MechStatus_rawdata_locked(self):
        status = CHSesame2MechStatus(rawdata="60030080f3ff0002")

        assert status.getBatteryPrecentage() == 100.0
        assert status.getBatteryVoltage() == 6.0809384164222875
        assert status.getPosition() == 65523
        assert status.getRetCode() == 0
        assert status.getTarget() == 32768
        assert status.isInLockRange()
        assert not status.isInUnlockRange()
        assert (
            str(status)
            == "CHSesame2MechStatus(Battery=100% (6.08V), isInLockRange=True, isInUnlockRange=False, Position=65523)"
        )

    def test_CHSesame2MechStatus_rawdata_unlocked(self):
        status = CHSesame2MechStatus(rawdata="5c030503e3020004")

        assert status.getBatteryPrecentage() == 100.0
        assert status.getBatteryVoltage() == 6.052785923753666
        assert status.getPosition() == 739
        assert status.getRetCode() == 0
        assert status.getTarget() == 773
        assert not status.isInLockRange()
        assert status.isInUnlockRange()
        assert (
            str(status)
            == "CHSesame2MechStatus(Battery=100% (6.05V), isInLockRange=False, isInUnlockRange=True, Position=739)"
        )



from __future__ import annotations

import sys
from enum import Enum
from typing import Union


class CHProductModel(Enum):
    WM2 = {
        "deviceModel": "wm_2",
        "isLocker": False,
        "productType": 1,
        "deviceFactory": None,
    }
    SS2 = {
        "deviceModel": "sesame_2",
        "isLocker": True,
        "productType": 0,
        "deviceFactory": "CHSesame2",
    }

    def getByModel(model: str) -> Union[CHProductModel, None]:
        if not isinstance(model, str):
            raise ValueError("Invalid Model")
        return next(
            (e for e in list(CHProductModel) if e.value["deviceModel"] == model), None
        )

    def getByValue(val: int) -> Union[CHProductModel, None]:
        if not isinstance(val, int):
            raise ValueError("Invalid Value")
        return next(
            (e for e in list(CHProductModel) if e.value["productType"] == val), None
        )

    def deviceModel(self) -> str:
        return self.value["deviceModel"]

    def isLocker(self) -> bool:
        return self.value["isLocker"]

    def productType(self) -> int:
        return self.value["productType"]

    def deviceFactory(self) -> Union[type, None]:
        if self.value["deviceFactory"] is not None:
            return getattr(sys.modules["pysesame3.lock"], self.value["deviceFactory"])
        else:
            return None


class CHSesame2MechStatus:
    def __init__(self, rawdata: str = None, dictdata: dict = None) -> None:
        """Represents a mechanical status of a device.

        Args:
            rawdata (str): The raw `mechst` string for the device from AWS IoT Shadow.
            dictdata (str): The structured `mechst` data for the device from Web API.
        """
        if rawdata is not None:
            data = bytes.fromhex(rawdata)
            self._batteryVoltage = int.from_bytes(data[0:2], "little") * 7.2 / 1023
            self._target = int.from_bytes(data[2:4], "little")
            self._position = int.from_bytes(data[4:6], "little")
            self._retcode = data[6]
            self._isInLockRange = data[7] & 2 > 0
            self._isInUnlockRange = data[7] & 4 > 0
            self._isBatteryCritical = data[7] & 32 > 0
        elif dictdata is not None:
            data = dictdata
            self._batteryVoltage = data["batteryVoltage"]
            self._position = data["position"]
            self._isInLockRange = True if data["CHSesame2Status"] == "locked" else False
            self._isInUnlockRange = (
                True if data["CHSesame2Status"] == "unlocked" else False
            )
        else:
            raise ValueError("No Input")

    def __str__(self) -> str:
        return f"CHSesame2MechStatus(Battery={self.getBatteryPrecentage()}% ({self.getBatteryVoltage():.2f}V), isInLockRange={self.isInLockRange()}, isInUnlockRange={self.isInUnlockRange()}, Position={self.getPosition()})"

    def getBatteryPrecentage(self) -> int:
        """Returns battery status information as a percentage.

        Returns:
            int: Battery power left as a percentage.
        """
        list_vol = [6.0, 5.8, 5.7, 5.6, 5.4, 5.2, 5.1, 5.0, 4.8, 4.6]
        list_pct = [100.0, 50.0, 40.0, 32.0, 21.0, 13.0, 10.0, 7.0, 3.0, 0.0]
        cur_vol = self._batteryVoltage

        if cur_vol >= list_vol[0]:
            return 100
        elif cur_vol <= list_vol[-1]:
            return 0
        else:
            ret = 0
            i = 0
            while i < len(list_vol) - 1:
                if cur_vol > list_vol[i] or cur_vol <= list_vol[i + 1]:
                    i = i + 1
                    continue
                else:
                    f = (cur_vol - list_vol[i + 1]) / (list_vol[i] - list_vol[i + 1])
                    f3 = list_pct[i]
                    f4 = list_pct[i + 1]
                    ret = int(f4 + (f * (f3 - f4)))
                    break

            return ret

    def getBatteryVoltage(self) -> float:
        """Returns battery status information as a voltage.

        Returns:
            float: Battery power left as a voltage.
        """
        return self._batteryVoltage

    def getPosition(self) -> int:
        """Returns current potision.

        Returns:
            int: The current position (-32767~0~32767)
        """
        return self._position

    def getRetCode(self) -> int:
        """Returns a return code.

        Returns:
            int: The result for a locking/unlocking request.
        """
        try:
            return self._retcode
        except AttributeError:
            raise NotImplementedError("Not Implemented in Web API")

    def getTarget(self) -> int:
        """Returns target potision.

        Returns:
            int: The target position (-32767~0~32767)
        """
        try:
            return self._target
        except AttributeError:
            raise NotImplementedError("Not Implemented in Web API")

    def isInLockRange(self) -> bool:
        """Returns whether a device is currently locked.

        Returns:
            bool: `True` if it is locked, `False` if not.
        """
        return self._isInLockRange

    def isInUnlockRange(self) -> bool:
        """Returns whether a device is currently unlocked.

        Returns:
            bool: `True` if it is unlocked, `False` if not.
        """
        return self._isInUnlockRange



#!/usr/bin/env python

"""Tests for `pysesame3` package."""


import pytest

from pysesame3.history import CHSesame2History

from .utils import load_fixture


class TestCHSesame2History:
    @pytest.fixture(autouse=True)
    def _load_json(self):
        self._json_entries = load_fixture("history.json")

    def test_CHSesame2History_raises_exception_on_missing_arguments(self):
        with pytest.raises(TypeError):
            CHSesame2History()

    def test_CHSesame2History_raises_exception_on_unknown_event_type(self):
        with pytest.raises(AttributeError) as excinfo:
            CHSesame2History.EventType.UNDEFINEDEVENTFORTEST
        assert str(excinfo.value) == "UNDEFINEDEVENTFORTEST"

        with pytest.raises(ValueError) as excinfo:
            CHSesame2History.EventType(12345)
        assert "12345 is not a valid" in str(excinfo.value)

        with pytest.raises(ValueError) as excinfo:
            CHSesame2History(type=12345, timeStamp=1597492862, recordID=1)
        assert "12345 is not a valid" in str(excinfo.value)

    def test_CHSesame2History(self):
        h = CHSesame2History(**(self._json_entries[0]))

        assert h.to_dict() == {
            "recordID": 200,
            "timeStamp": "2021/06/07 17:33:20",
            "type": "webUnLock",
            "historyTag": "ThisIsTest",
        }

    def test_CHSesame2History_minimum(self):
        h = CHSesame2History(**(self._json_entries[1]))

        assert h.to_dict() == {
            "recordID": 201,
            "timeStamp": "2021/06/07 17:50:34",
            "type": "manualLocked",
            "historyTag": None,
        }



#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import pytest

from pysesame3.auth import CognitoAuth, WebAPIAuth


def test_WebAPIAuth_raises_exception_on_missing_arguments():
    with pytest.raises(TypeError):
        WebAPIAuth()


def test_WebAPIAuth_raises_exception_on_invalid_apikey():
    with pytest.raises(ValueError) as excinfo:
        WebAPIAuth(apikey="thisisfake")
    assert "length should be 40" in str(excinfo.value)


def test_CognitoAuth_raises_exception_on_missing_arguments():
    with pytest.raises(TypeError) as excinfo:
        CognitoAuth()
    assert "required positional argument" in str(excinfo.value)

    with pytest.raises(TypeError) as excinfo:
        CognitoAuth(apikey="FAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKE")
    assert "required positional argument" in str(excinfo.value)

    with pytest.raises(TypeError) as excinfo:
        CognitoAuth(client_id="us-east-1:aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee")
    assert "required positional argument" in str(excinfo.value)


def test_CognitoAuth_raises_exception_on_invalid_apikey():
    with pytest.raises(ValueError) as excinfo:
        CognitoAuth(
            apikey="fake", client_id="us-east-1:aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee"
        )
    assert "length should be 40" in str(excinfo.value)



from enum import Enum, auto

OFFICIALAPI_URL = "https://app.candyhouse.co/api/sesame2"
APIGW_URL = "https://jhcr1i3ecb.execute-api.ap-northeast-1.amazonaws.com/prod"
IOT_EP = "a3i4hui4gxwoo8-ats.iot.ap-northeast-1.amazonaws.com"


class AuthType(Enum):
    WebAPI = auto()
    SDK = auto()



import base64
from datetime import datetime
from enum import IntEnum


class CHSesame2History:
    """A class for representing a historical event of SESAME devices.

    Attributes:
        type (int): Type of event as defined in `CHSesame2History.EventType` class.
        timeStamp (int): Timestamp in milliseconds since 1970/1/1 00:00:00.
        recordID (int): Unique ID in a device.
        historyTag (bytes): Tag on the key that triggered this event.
    """

    class EventType(IntEnum):
        none = 0
        bleLock = 1
        bleUnLock = 2
        timeChanged = 3
        autoLockUpdated = 4
        mechSettingUpdated = 5
        autoLock = 6
        manualLocked = 7
        manualUnlocked = 8
        manualElse = 9
        driveLocked = 10
        driveUnlocked = 11
        driveFailed = 12
        bleAdvParameterUpdated = 13
        wm2Lock = 14
        wm2UnLock = 15
        webLock = 16
        webUnLock = 17

    def __init__(
        self, type: int, timeStamp: int, recordID: int, historyTag: bytes = None
    ) -> None:
        self.event_type = CHSesame2History.EventType(type)
        self.timestamp = datetime.fromtimestamp(timeStamp / 1000)
        self.record_id = recordID
        self.historytag = historyTag

    def to_dict(self) -> dict:
        """Returns a dict representation of an object.

        Returns:
            dist: The dict representation of the object.
        """
        return {
            "recordID": self.record_id,
            "timeStamp": self.timestamp.strftime("%Y/%m/%d %H:%M:%S"),
            "type": self.event_type.name,
            "historyTag": base64.b64decode(self.historytag).decode("utf-8")
            if self.historytag is not None
            else None,
        }



from __future__ import annotations

import uuid
from typing import TYPE_CHECKING, Union

if TYPE_CHECKING:
    from pysesame3.auth import CognitoAuth, WebAPIAuth
    from pysesame3.helper import CHProductModel


class CHDevices:
    def __init__(self, authenticator: Union[WebAPIAuth, CognitoAuth]):
        """Generic Implementation for Candyhouse products.

        Args:
            authenticator (Union[WebAPIAuth, CognitoAuth]): The authenticator for the device
        """
        self._authenticator = authenticator
        self._deviceId = None
        self._productModel = None

    @property
    def authenticator(self) -> WebAPIAuth:
        return self._authenticator

    @property
    def deviceId(self) -> str:
        """Returns a device id of a specific device.

        Returns:
            str: The UUID of the device.
        """
        if self._deviceId is not None:
            return str(self._deviceId).upper()
        else:
            return None

    @property
    def productModel(self) -> CHProductModel:
        """Returns a model information of a specific device.

        Returns:
            CHProductModel: The product model of the device.
        """
        return self._productModel

    def setDeviceId(self, id: Union[uuid.UUID, str]) -> None:
        """Sets a device id of a specific device.

        Args:
            id (Union[uuid.UUID, str]): The UUID of the device.

        Raises:
            ValueError: If `id` is invalid.
        """
        if isinstance(id, str):
            id = uuid.UUID(id)
        elif not isinstance(id, uuid.UUID):
            raise ValueError("Invalid UUID")
        self._deviceId = id

    def setProductModel(self, model: CHProductModel) -> None:
        """Sets a model information of a specific device.

        Args:
            model (CHProductModel): The product model of the device.

        Raises:
            ValueError: If `model` is invalid.
        """
        if type(model).__name__ != "CHProductModel":
            raise ValueError("Invalid CHProductModel")
        self._productModel = model


class SesameLocker(CHDevices):
    def __init__(self, authenticator: Union[WebAPIAuth, CognitoAuth]):
        """Generic Implementation for Candyhouse smart locks.

        Args:
            authenticator (Union[WebAPIAuth, CognitoAuth]): The authenticator for the device
        """
        super().__init__(authenticator)
        self._mechStatus = None
        self._secretKey = None
        self._sesame2PublicKey = None

    def getDeviceUUID(self) -> str:
        """Gets a device UUID of a specific device.

        Returns:
            str: The UUID of the device.
        """
        return self.deviceId

    def getSecretKey(self) -> str:
        """Gets a secret key of a specific device.

        Returns:
            str: The secret key for controlling the device.
        """
        return self._secretKey

    def getSesame2PublicKey(self) -> str:
        """Gets a public key of a specific device.

        Returns:
            str: The public key of the device.
        """
        return self._sesame2PublicKey

    def setDeviceUUID(self, id: Union[uuid.UUID, str]) -> None:
        """Sets a device UUID of a specific device.

        Args:
            id (Union[uuid.UUID, str]): The UUID of the device.
        """
        self.setDeviceId(id)

    def setSecretKey(self, key: str) -> None:
        """Sets a secret key for a specific device.

        Args:
            key (str): The secret key for controlling the device.

        Raises:
            ValueError: If `key` is invalid.
        """
        if not isinstance(key, str):
            raise ValueError("Invalid SecretKey - should be string.")
        if len(key) != 32:
            raise ValueError("Invalid SecretKey - length should be 32.")
        self._secretKey = key

    def setSesame2PublicKey(self, key: str) -> None:
        """Sets a public key of a specific device.

        Args:
            key (str): The public key of the device.

        Raises:
            ValueError: If `key` is invalid.
        """
        if not isinstance(key, str):
            raise ValueError("Invalid Sesame2PublicKey")
        self._sesame2PublicKey = key

    def __str__(self) -> str:
        """Returns a string representation of an object.

        Returns:
            str: The string representation of the object.
        """
        return f"SesameLocker(deviceUUID={self.getDeviceUUID()}, deviceModel={self.productModel}, sesame2PublicKey={self.getSesame2PublicKey()})"



import sys

try:
    import boto3
    from requests_aws4auth import AWS4Auth
except ImportError:
    # Optional deps
    pass
import requests

from .const import IOT_EP, AuthType


class WebAPIAuth(requests.auth.AuthBase):
    def __init__(self, apikey: str):
        """Generic Implementation for Web API Authentication.

        Args:
            apikey (str): API Key
        """
        self._apikey = apikey

        if len(self._apikey) != 40:
            raise ValueError("Invalid API Key - length should be 40.")

    @property
    def login_method(self):
        return AuthType.WebAPI

    def __call__(self, r):
        r.headers["x-api-key"] = self._apikey
        return r


class CognitoAuth:
    def __init__(self, apikey: str, client_id: str):
        """Generic Implementation for Web API Authentication.

        Args:
            apikey (str): API Key
            client_id (str): Client ID
        """
        self._apikey = apikey
        self._client_id = client_id

        if len(self._apikey) != 40:
            raise ValueError("Invalid API Key - length should be 40.")

        if "AWSIoTPythonSDK" not in sys.modules or "certifi" not in sys.modules:
            raise RuntimeError(
                "Failed to load AWSIoTPythonSDK or certifi. Did you run `pip install pysesame3[cognito]`?"
            )

    @property
    def login_method(self) -> AuthType:
        return AuthType.SDK

    @property
    def client_id(self) -> str:
        return self._client_id

    def authenticate(self):
        region_name = self._client_id.split(":")[0]

        cognitoIdentityClient = boto3.client(
            "cognito-identity", region_name=region_name
        )
        temporaryIdentityId = cognitoIdentityClient.get_id(
            IdentityPoolId=self._client_id
        )
        identityID = temporaryIdentityId["IdentityId"]

        temporaryCredentials = cognitoIdentityClient.get_credentials_for_identity(
            IdentityId=identityID
        )
        access_key_id = temporaryCredentials["Credentials"]["AccessKeyId"]
        secret_key = temporaryCredentials["Credentials"]["SecretKey"]
        session_token = temporaryCredentials["Credentials"]["SessionToken"]

        return (access_key_id, secret_key, session_token)

    def __call__(self, r):
        (access_key_id, secret_key, session_token) = self.authenticate()

        if IOT_EP in r.url:
            service = "iotdata"
        else:
            r.headers["x-api-key"] = self._apikey
            service = "execute-api"

        auth = AWS4Auth(
            access_key_id,
            secret_key,
            "ap-northeast-1",
            service,
            session_token=session_token,
        )
        r = auth(r)

        return r



#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import pytest
import requests_mock
from moto import mock_cognitoidentity

from pysesame3.auth import CognitoAuth, WebAPIAuth
from pysesame3.history import CHSesame2History
from pysesame3.lock import CHSesame2, CHSesame2ShadowStatus

from .utils import load_fixture


@pytest.fixture(autouse=True)
def mock_requests():
    with requests_mock.Mocker() as mock:
        mock.get(
            "https://app.candyhouse.co/api/sesame2/126d3d66-9222-4e5a-bcde-0c6629d48d43",
            json=load_fixture("lock_get_locked.json"),
        )

        mock.get(
            "https://app.candyhouse.co/api/sesame2/e0e56521-63d8-4da5-ba4b-c4a6a5e353f1",
            json=load_fixture("lock_get_unlocked.json"),
        )

        mock.post(
            "https://app.candyhouse.co/api/sesame2/126d3d66-9222-4e5a-bcde-0c6629d48d43/cmd"
        )

        mock.post(
            "https://app.candyhouse.co/api/sesame2/e0e56521-63d8-4da5-ba4b-c4a6a5e353f1/cmd"
        )

        mock.get(
            "https://a3i4hui4gxwoo8-ats.iot.ap-northeast-1.amazonaws.com/things/sesame2/shadow?name=126D3D66-9222-4E5A-BCDE-0C6629D48D43",
            json=load_fixture("lock_shadow_locked.json"),
        )

        mock.get(
            "https://a3i4hui4gxwoo8-ats.iot.ap-northeast-1.amazonaws.com/things/sesame2/shadow?name=E0E56521-63D8-4DA5-BA4B-C4A6A5E353F1",
            json=load_fixture("lock_shadow_unlocked.json"),
        )

        mock.post(
            "https://jhcr1i3ecb.execute-api.ap-northeast-1.amazonaws.com/prod/device/v1/iot/sesame2/126D3D66-9222-4E5A-BCDE-0C6629D48D43"
        )

        mock.post(
            "https://jhcr1i3ecb.execute-api.ap-northeast-1.amazonaws.com/prod/device/v1/iot/sesame2/E0E56521-63D8-4DA5-BA4B-C4A6A5E353F1"
        )

        mock.get(
            "https://app.candyhouse.co/api/sesame2/126d3d66-9222-4e5a-bcde-0c6629d48d43/history?page=0&lg=10",
            json=load_fixture("history.json"),
        )

        mock.get(
            "https://jhcr1i3ecb.execute-api.ap-northeast-1.amazonaws.com/prod/device/v1/sesame2/126D3D66-9222-4E5A-BCDE-0C6629D48D43/history",
            json=load_fixture("history.json"),
        )
        yield mock


@pytest.fixture()
def mock_cloud():
    cl = WebAPIAuth(apikey="FAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKE")
    yield cl


@pytest.fixture()
def mock_cloud_cognito():
    with mock_cognitoidentity():
        cl = CognitoAuth(
            apikey="FAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKE",
            client_id="ap-northeast-1:fakefakefake",
        )
        yield cl


class TestCHSesame2Broken:
    def test_CHSesame2_raises_exception_on_missing_arguments(self):
        with pytest.raises(TypeError):
            CHSesame2()

        with pytest.raises(TypeError):
            CHSesame2(mock_cloud)

    def test_CHSesame2_raises_exception_on_invalid_arguments(self):
        with pytest.raises(ValueError):
            key = {
                "device_uuid": "FAKEUUID",
                "secret_key": "0b3e5f1665e143b59180c915fa4b06d9",
            }
            CHSesame2(mock_cloud, **key)

        with pytest.raises(ValueError):
            key = {
                "device_uuid": "126D3D66-9222-4E5A-BCDE-0C6629D48D43",
                "secret_key": "FAKE_SECRET_KEY",
            }
            CHSesame2(mock_cloud, **key)


class TestCHSesame2:
    @pytest.fixture(autouse=True)
    def _initialize_key(self, mock_cloud):
        key = {
            "device_uuid": "126D3D66-9222-4E5A-BCDE-0C6629D48D43",
            "secret_key": "0b3e5f1665e143b59180c915fa4b06d9",
        }
        self.key_locked = CHSesame2(mock_cloud, **key)

        key = {
            "device_uuid": "E0E56521-63D8-4DA5-BA4B-C4A6A5E353F1",
            "secret_key": "0b3e5f1665e143b59180c915fa4b06d9",
        }
        self.key_unlocked = CHSesame2(mock_cloud, **key)

    def test_CHSesame2(self):
        assert (
            str(self.key_locked)
            == "CHSesame2(deviceUUID=126D3D66-9222-4E5A-BCDE-0C6629D48D43, deviceModel=None, sesame2PublicKey=None, mechStatus=CHSesame2MechStatus(Battery=67% (5.87V), isInLockRange=True, isInUnlockRange=False, Position=11))"
        )

    def test_CHSesame2_getDeviceShadowStatus(self):
        assert self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        assert (
            self.key_unlocked.getDeviceShadowStatus()
            == CHSesame2ShadowStatus.UnlockedWm
        )

    def test_SesameCloud_getHistoryEntries(self):
        entries = self.key_locked.historyEntries

        assert isinstance(entries, list)
        assert len(entries) == 2
        assert isinstance(entries[0], CHSesame2History)
        assert isinstance(entries[1], CHSesame2History)

    def test_CHSesame2_setDeviceShadowStatus_toggle(self):
        assert self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        self.key_locked.setDeviceShadowStatus(CHSesame2ShadowStatus.UnlockedWm)
        assert (
            self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm
        )

    def test_CHSesame2_setDeviceShadowStatus_raises_exception_on_invalid_status(self):
        with pytest.raises(ValueError):
            self.key_locked.setDeviceShadowStatus("INVALID")

    def test_CHSesame2_lock_fails_HTTP_requests(self):
        with requests_mock.Mocker() as mock:
            mock.post(
                "https://app.candyhouse.co/api/sesame2/e0e56521-63d8-4da5-ba4b-c4a6a5e353f1/cmd",
                status_code=500,
            )
            assert not self.key_unlocked.lock()
            assert (
                self.key_unlocked.getDeviceShadowStatus()
                == CHSesame2ShadowStatus.UnlockedWm
            )

    def test_CHSesame2_lock(self):
        assert self.key_unlocked.lock()
        assert (
            self.key_unlocked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        )

    def test_CHSesame2_unlock(self):
        assert self.key_locked.unlock()
        assert (
            self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm
        )

    def test_CHSesame2_toggle(self):
        assert self.key_locked.toggle()
        assert (
            self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm
        )
        assert self.key_locked.toggle()
        assert self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm

        assert self.key_unlocked.toggle()
        assert (
            self.key_unlocked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        )
        assert self.key_unlocked.toggle()
        assert (
            self.key_unlocked.getDeviceShadowStatus()
            == CHSesame2ShadowStatus.UnlockedWm
        )


class TestCHSesame2Cognito:
    @pytest.fixture(autouse=True)
    def _initialize_key(self, mock_cloud_cognito):
        key = {
            "device_uuid": "126D3D66-9222-4E5A-BCDE-0C6629D48D43",
            "secret_key": "0b3e5f1665e143b59180c915fa4b06d9",
        }
        self.key_locked = CHSesame2(mock_cloud_cognito, **key)

        key = {
            "device_uuid": "E0E56521-63D8-4DA5-BA4B-C4A6A5E353F1",
            "secret_key": "0b3e5f1665e143b59180c915fa4b06d9",
        }
        self.key_unlocked = CHSesame2(mock_cloud_cognito, **key)

    def test_CHSesame2(self):
        assert (
            str(self.key_locked)
            == "CHSesame2(deviceUUID=126D3D66-9222-4E5A-BCDE-0C6629D48D43, deviceModel=None, sesame2PublicKey=None, mechStatus=CHSesame2MechStatus(Battery=100% (6.11V), isInLockRange=True, isInUnlockRange=False, Position=29))"
        )

    def test_CHSesame2_getDeviceShadowStatus(self):
        assert self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        assert (
            self.key_unlocked.getDeviceShadowStatus()
            == CHSesame2ShadowStatus.UnlockedWm
        )

    def test_SesameCloud_getHistoryEntries(self):
        entries = self.key_locked.historyEntries

        assert isinstance(entries, list)
        assert len(entries) == 2
        assert isinstance(entries[0], CHSesame2History)
        assert isinstance(entries[1], CHSesame2History)

    def test_CHSesame2_setDeviceShadowStatus_toggle(self):
        assert self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        self.key_locked.setDeviceShadowStatus(CHSesame2ShadowStatus.UnlockedWm)
        assert (
            self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm
        )

    def test_CHSesame2_setDeviceShadowStatus_raises_exception_on_invalid_status(self):
        with pytest.raises(ValueError):
            self.key_locked.setDeviceShadowStatus("INVALID")

    def test_CHSesame2_lock_fails_HTTP_requests(self):
        with requests_mock.Mocker() as mock:
            mock.post(
                "https://jhcr1i3ecb.execute-api.ap-northeast-1.amazonaws.com/prod/device/v1/iot/sesame2/E0E56521-63D8-4DA5-BA4B-C4A6A5E353F1",
                status_code=500,
            )
            assert not self.key_unlocked.lock()
            assert (
                self.key_unlocked.getDeviceShadowStatus()
                == CHSesame2ShadowStatus.UnlockedWm
            )

    def test_CHSesame2_lock(self):
        assert self.key_unlocked.lock()
        assert (
            self.key_unlocked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        )

    def test_CHSesame2_unlock(self):
        assert self.key_locked.unlock()
        assert (
            self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm
        )

    def test_CHSesame2_toggle(self):
        assert self.key_locked.toggle()
        assert (
            self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm
        )
        assert self.key_locked.toggle()
        assert self.key_locked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm

        assert self.key_unlocked.toggle()
        assert (
            self.key_unlocked.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm
        )
        assert self.key_unlocked.toggle()
        assert (
            self.key_unlocked.getDeviceShadowStatus()
            == CHSesame2ShadowStatus.UnlockedWm
        )



#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import uuid

import pytest

from pysesame3.auth import WebAPIAuth
from pysesame3.device import CHDevices, SesameLocker
from pysesame3.helper import CHProductModel


@pytest.fixture(autouse=True)
def cl():
    yield WebAPIAuth(apikey="FAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKE")


class TestCHDevices:
    def test_CHDevices_deviceId_raises_exception_on_invalid_uuid(self):
        d = CHDevices(cl)

        with pytest.raises(ValueError):
            d.setDeviceId("INVALID-UUID")

        with pytest.raises(ValueError):
            d.setDeviceId(12345)

    def test_CHDevices_deviceId(self):
        d = CHDevices(cl)

        assert d.deviceId is None

        test_uuid = "42918AD1-8154-4AFF-BD1F-F0CDE88A8DE1"
        assert d.setDeviceId(test_uuid) is None
        assert d.deviceId == test_uuid

        test_uuid2 = uuid.UUID("42918AD1-8154-4AFF-BD1F-F0CDE88A8DE1")
        assert d.setDeviceId(test_uuid2) is None
        assert d.deviceId == str(test_uuid2).upper()

    def test_CHDevices_productModel_raises_exception_on_invalid_uuid(self):
        d = CHDevices(cl)

        with pytest.raises(ValueError):
            d.setProductModel("INVALID-PRODUCT")

    def test_CHDevices_productModel(self):
        d = CHDevices(cl)

        assert d.productModel is None

        test_model = CHProductModel.SS2
        assert d.setProductModel(test_model) is None
        assert d.productModel == test_model


class TestSesameLocker:
    def test_SesameLocker_deviceUUID_raises_exception_on_invalid_uuid(self):
        d = SesameLocker(cl)

        with pytest.raises(ValueError):
            d.setDeviceUUID("INVALID-UUID")

    def test_CHDevices_deviceUUID(self):
        d = SesameLocker(cl)

        assert d.getDeviceUUID() is None

        test_uuid = "42918AD1-8154-4AFF-BD1F-F0CDE88A8DE1"
        assert d.setDeviceUUID(test_uuid) is None
        assert d.getDeviceUUID() == test_uuid

        test_uuid2 = uuid.UUID("42918AD1-8154-4AFF-BD1F-F0CDE88A8DE1")
        assert d.setDeviceUUID(test_uuid2) is None
        assert d.getDeviceUUID() == str(test_uuid2).upper()

    def test_SesameLocker_secretKey_raises_exception_on_invalid_value(self):
        d = SesameLocker(cl)

        with pytest.raises(ValueError) as excinfo:
            d.setSecretKey(123)
        assert "should be string" in str(excinfo.value)

        with pytest.raises(ValueError) as excinfo:
            d.setSecretKey("FAKE")
        assert "length should be 32" in str(excinfo.value)

    def test_CHDevices_secretKey(self):
        d = SesameLocker(cl)

        assert d.getSecretKey() is None

        secret = "FAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKE"

        assert d.setSecretKey(secret) is None
        assert d.getSecretKey() == secret

    def test_SesameLocker_sesame2PublicKey_raises_exception_on_invalid_value(self):
        d = SesameLocker(cl)

        with pytest.raises(ValueError):
            d.setSesame2PublicKey(123)

    def test_CHDevices_sesame2PublicKey(self):
        d = SesameLocker(cl)

        assert d.getSesame2PublicKey() is None

        pubkey = "TestPubKey"

        assert d.setSesame2PublicKey(pubkey) is None
        assert d.getSesame2PublicKey() == pubkey

    def test_CHDevices(self):
        d = SesameLocker(cl)

        test_uuid = "42918AD1-8154-4AFF-BD1F-F0CDE88A8DE1"
        d.setDeviceUUID(test_uuid)

        test_model = CHProductModel.SS2
        d.setProductModel(test_model)

        secret = "FAKEFAKEFAKEFAKEFAKEFAKEFAKEFAKE"
        d.setSecretKey(secret)

        pubkey = "TestPubKey"
        d.setSesame2PublicKey(pubkey)

        assert (
            str(d)
            == "SesameLocker(deviceUUID=42918AD1-8154-4AFF-BD1F-F0CDE88A8DE1, deviceModel=CHProductModel.SS2, sesame2PublicKey=TestPubKey)"
        )



from __future__ import annotations

import json
from enum import Enum, IntEnum, auto
from typing import TYPE_CHECKING, Callable, Union

try:
    import certifi
    from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTClient
except ImportError:
    # Optional deps
    pass

from pysesame3.cloud import SesameCloud
from pysesame3.const import IOT_EP, AuthType
from pysesame3.device import SesameLocker
from pysesame3.helper import CHSesame2MechStatus

if TYPE_CHECKING:
    from pysesame3.auth import CognitoAuth, WebAPIAuth
    from pysesame3.history import CHSesame2History


class CHSesame2CMD(IntEnum):
    LOCK = 82
    UNLOCK = 83
    TOGGLE = 88


class CHSesame2ShadowStatus(Enum):
    LockedWm = auto()
    UnlockedWm = auto()
    MovedWm = auto()


class CHSesame2(SesameLocker):
    def __init__(
        self,
        authenticator: Union[WebAPIAuth, CognitoAuth],
        device_uuid: str,
        secret_key: str,
    ) -> None:
        """SESAME3 Device Specific Implementation.

        Args:
            authenticator (Union[WebAPIAuth, CognitoAuth]):
            device_uuid (str): The UUID of the device
            secret_key (str): The secret key of the device
        """
        super().__init__(authenticator)

        self.setDeviceUUID(device_uuid)
        self.setSecretKey(secret_key)

        # Initial sync of `self._deviceShadowStatus`
        self.mechStatus

    @property
    def mechStatus(self) -> CHSesame2MechStatus:
        """Returns a mechanical status of a device.

        Returns:
            CHSesame2MechStatus: Current mechanical status of the device.
        """
        status = SesameCloud(self).getMechStatus()

        if status.isInLockRange():
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.LockedWm)
        else:
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.UnlockedWm)

        return status

    def _iot_shadow_callback(self, client, userdata, message) -> None:
        """Example Callback for updated shadows.

        Args:
            message ([type]): The device shadow
        """
        shadow = json.loads(message.payload)
        status = CHSesame2MechStatus(rawdata=shadow["state"]["reported"]["mechst"])
        if status.isInLockRange():
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.LockedWm)
        else:
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.UnlockedWm)

    def subscribeMechStatus(self, callback: Callable[..., None] = None) -> bool:
        """Subscribes to a topic at AWS IoT

        Args:
            callback (Callable[..., None], optional): The registered callback will be executed once an update is delivered. Defaults to `_iot_shadow_callback`.

        Raises:
            NotImplementedError: If the authenticator is not `AuthType.SDK`.

        Returns:
            bool: [description]
        """
        if self.authenticator.login_method != AuthType.SDK:
            raise NotImplementedError("This feature is not suppoted by the Web API.")

        callback = callback or self._iot_shadow_callback

        (access_key_id, secret_key, session_token) = self.authenticator.authenticate()

        c = AWSIoTMQTTClient(self.authenticator.client_id, useWebsocket=True)
        c.configureEndpoint(IOT_EP, 443)
        c.configureCredentials(certifi.where())
        c.configureIAMCredentials(access_key_id, secret_key, session_token)
        c.connect()
        c.subscribe(
            "$aws/things/sesame2/shadow/name/{}/update/accepted".format(
                self.getDeviceUUID()
            ),
            1,
            callback,
        )

    @property
    def historyEntries(self) -> list[CHSesame2History]:
        """Returns the history of all events with a device.

        Returns:
            list[CHSesame2History]: A list of events.
        """
        return SesameCloud(self).getHistoryEntries()

    def getDeviceShadowStatus(self) -> CHSesame2ShadowStatus:
        """Returns a cached shadow status of a device.
        In order to refresh the shadow, run `mechStatus`.

        Returns:
            CHSesame2ShadowStatus: Shadow (assumed) status of the device.
        """
        return self._deviceShadowStatus

    def setDeviceShadowStatus(self, status: CHSesame2ShadowStatus) -> None:
        """Sets a shadow status of a device.

        Args:
            status (CHSesame2ShadowStatus): Desired shadow (assumed) status

        Raises:
            ValueError: If `status` is invalid.
        """
        if not isinstance(status, CHSesame2ShadowStatus):
            raise ValueError("Invalid CHSesame2ShadowStatus")
        self._deviceShadowStatus = status

    def lock(self, history_tag: str = "pysesame3") -> bool:
        """Locking.

        Args:
            history_tag (str): The key tag to sent when locking and unlocking. Defaults to `pysesame3`.

        Returns:
            bool: `True` if it is successfully locked, `False` if not.
        """
        result = SesameCloud(self).sendCmd(CHSesame2CMD.LOCK, history_tag)
        if result:
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.LockedWm)
        return result

    def unlock(self, history_tag: str = "pysesame3") -> bool:
        """Unlocking.

        Args:
            history_tag (str): The key tag to sent when locking and unlocking. Defaults to `pysesame3`.

        Returns:
            bool: `True` if it is successfully unlocked, `False` if not.
        """
        result = SesameCloud(self).sendCmd(CHSesame2CMD.UNLOCK, history_tag)
        if result:
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.UnlockedWm)
        return result

    def toggle(self, history_tag: str = "pysesame3") -> bool:
        """Toggle.

        Args:
            history_tag (str): The key tag to sent when locking and unlocking. Defaults to `pysesame3`.

        Returns:
            bool: `True` if it is successfully toggled, `False` if not.
        """
        if self.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm:
            return self.unlock(history_tag)
        elif self.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm:
            return self.lock(history_tag)

    def __str__(self) -> str:
        """Return a string representation of an object.

        Returns:
            str: The string representation of the object.
        """
        return f"CHSesame2(deviceUUID={self.getDeviceUUID()}, deviceModel={self.productModel}, sesame2PublicKey={self.getSesame2PublicKey()}, mechStatus={self.mechStatus})"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pysesame3', 'tests']

package_data = \
{'': ['*'], 'tests': ['fixtures/*']}

install_requires = \
['pycryptodome>=3.10.1,<4.0.0', 'requests>=2.25.1,<3.0.0']

extras_require = \
{'cognito': ['AWSIoTPythonSDK>=1.4.9,<2.0.0',
             'boto3>=1.17.94,<2.0.0',
             'certifi',
             'requests-aws4auth>=1.1.1,<2.0.0'],
 'doc': ['livereload>=2.6.3,<3.0.0',
         'mkdocs>=1.1.2,<2.0.0',
         'mkdocstrings>=0.13.6,<0.14.0',
         'mkdocs-autorefs>=0.2.1,<0.3.0',
         'mkdocs-include-markdown-plugin>=3.1.3,<4.0.0',
         'mkdocs-material>=7.1.8,<8.0.0']}

setup_kwargs = {
    'name': 'pysesame3',
    'version': '0.3.0',
    'description': 'Unofficial library to communicate with Sesame smart locks.',
    'long_description': '# pysesame3\n\n_Unofficial Python Library to communicate with Sesame smart locks made by CANDY HOUSE, Inc._\n\n[![PyPI](https://img.shields.io/pypi/v/pysesame3)](https://pypi.python.org/pypi/pysesame3)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pysesame3)\n![GitHub Workflow Status (branch)](https://img.shields.io/github/workflow/status/mochipon/pysesame3/dev%20workflow/main)\n[![Documentation Status](https://readthedocs.org/projects/pysesame3/badge/?version=latest)](https://pysesame3.readthedocs.io/en/latest/?badge=latest)\n[![codecov](https://codecov.io/gh/mochipon/pysesame3/branch/main/graph/badge.svg?token=2Y7OPZTILT)](https://codecov.io/gh/mochipon/pysesame3)\n![PyPI - License](https://img.shields.io/pypi/l/pysesame3)\n\n* Free software: MIT license\n* Documentation: [https://pysesame3.readthedocs.io](https://pysesame3.readthedocs.io)\n\n## Features\n\nPlease note that `pysesame3` can only control [SESAME 3](https://jp.candyhouse.co/products/sesame3) at this moment.\n\n* Retrive a list of SESAME locks that the user is authorized to use.\n* Retrive a status of a SESAME lock (locked, handle position, etc.).\n* Retrive recent events (locked, unlocked, etc.) associated with a lock.\n* Needless to say, locking and unlocking!\n\n## Usage\n\nPlease take a look at [the documentation](https://pysesame3.readthedocs.io/en/latest/usage/).\n\n## Credits & Thanks\n\n* A huge thank you to all who assisted with [CANDY HOUSE](https://jp.candyhouse.co/).\n* This project was inspired and based on [tchellomello/python-ring-doorbell](https://github.com/tchellomello/python-ring-doorbell) and [snjoetw/py-august](https://github.com/snjoetw/py-august).\n',
    'author': 'Masaki Tagawa',
    'author_email': 'masaki@tagawa.email',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mochipon/pysesame3',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import pytest
import requests_mock
from moto import mock_cognitoidentity

from pysesame3.cloud import SesameCloud

from .utils import load_fixture


@pytest.fixture(autouse=True)
def mock_requests():
    with requests_mock.Mocker() as mock:
        mock.get(
            "https://app.candyhouse.co/api/sesame2/FAKEUUID/history?page=0&lg=10",
            json=load_fixture("history.json"),
        )

        mock.get(
            "https://jhcr1i3ecb.execute-api.ap-northeast-1.amazonaws.com/prod/device/v1/sesame2/FAKEUUID/history",
            json=load_fixture("history.json"),
        )
        yield mock


class TestSesameCloudBroken:
    def test_SesameCloud_raises_exception_on_authenticator_missing(self):
        with pytest.raises(TypeError) as excinfo:
            SesameCloud()
        assert "missing 1 required positional argument" in str(excinfo.value)
